# Reference

<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

### Defined types

#### Public Defined types

* [`collections::append`](#collections--append): Append an item of any type to a collection.
* [`collections::create`](#collections--create): Create a new collection
* [`collections::debug_executor`](#collections--debug_executor): Collections debugging - print the set of items passed in
* [`collections::file`](#collections--file): Create a file which allows many resources to contribute content.
* [`collections::register_action`](#collections--register_action): Register a defined type to be run for each item in a collection.
* [`collections::register_executor`](#collections--register_executor): Register a defined type to be run once, for all items in a collection.
* [`collections::tap`](#collections--tap): Print the items received for debugging

#### Private Defined types

* `collections::checkpoint`: An empty resource recording stages within a collection
* `collections::commit`: The resource that calls actions and executors upon a collection
* `collections::file::writer`: Write collected data to a file
* `collections::iterator`: Wrapper resource to allow predictable resource ordering

### Functions

* [`collections::deep_merge`](#collections--deep_merge): Wrapper for the Ruby deep_merge gem

## Defined types

### <a name="collections--append"></a>`collections::append`

Append an item of any type to a collection.

#### Examples

##### 

```puppet
collections::append { 'User foo is an admin':
  target => 'users::admin-users',
  data   => {
    username => 'foo',
    uid      => 1001,
  },
}
```

#### Parameters

The following parameters are available in the `collections::append` defined type:

* [`target`](#-collections--append--target)
* [`data`](#-collections--append--data)

##### <a name="-collections--append--target"></a>`target`

Data type: `String[1]`

The target collection, created by `collection::create`

##### <a name="-collections--append--data"></a>`data`

Data type: `Any`

The item to be added to the target collection

### <a name="collections--create"></a>`collections::create`

Create a collection and the unerlying `collection::iterator` resource which powers it.

#### Examples

##### 

```puppet
collections::create { 'collection-name':
}

collections::create { 'Operations for all users':
  target   => 'users::all',
  defaults => {
    parent => '/home'
  }
}
```

#### Parameters

The following parameters are available in the `collections::create` defined type:

* [`target`](#-collections--create--target)
* [`defaults`](#-collections--create--defaults)
* [`initial_items`](#-collections--create--initial_items)

##### <a name="-collections--create--target"></a>`target`

Data type: `String[1]`

The name of this collection. You must pass this target name to all resources that
act upon or work with this collection.

Default value: `$title`

##### <a name="-collections--create--defaults"></a>`defaults`

Data type: `Hash[String, Any]`

Parameters that will be used as defaults for any resources generated by this collection.
When a resource is created, these will be passed in as default parameters.

Default value: `{}`

##### <a name="-collections--create--initial_items"></a>`initial_items`

Data type: `Array[Any]`

A set of items to begine the collection with. This should be an array of data that could
be added one at a time with `collection::append`.

Default value: `[]`

### <a name="collections--debug_executor"></a>`collections::debug_executor`

This provides an example of an executor and possibly a useful debugging tool.
In normal use, you would never create a resource of this type manually, it
would instead be created by `collections::register_executor`.

* **See also**
  * collections::register_executor

#### Examples

##### 

```puppet

collections::register_executor { 'Debug: Print all items added to the collection':
  target   => 'an-exiting-collection',
  resource => 'collections::debug_executor',
}
```

#### Parameters

The following parameters are available in the `collections::debug_executor` defined type:

* [`target`](#-collections--debug_executor--target)
* [`items`](#-collections--debug_executor--items)

##### <a name="-collections--debug_executor--target"></a>`target`

Data type: `String[1]`

Passed in by `collections::commit` when creating this resource. It indicates the
name of the collection that it was spawned from, to allow any

##### <a name="-collections--debug_executor--items"></a>`items`

Data type: `Array[Any]`

Passed in by `collections::commit` when creating this resource. It contains an
array of all the items added to this collection using `collections::append`

### <a name="collections--file"></a>`collections::file`

This is a convenience function that uses a collection to handle the problem
of allowing portions of a file to be defined in many places. The file will
be generate from a template, with data collated from any number of
`collection::file::fragment` resources.

The output of the collated resources will be a variable `$data` (or `@data`
within erb templates). As a convenience, a small set of templates are
predefined within the collections module to suit a few use cases:

#### `collections/concat.erb`
This template allows constructing a file from multiple content blocks, with
ordering based upon an optional `order` key. Fragments should contain a
`content` key. Order will default to 1000 if it is not supplied.

#### `collections/yaml.erb`
This template will output the collected data as a YAML document

#### `collections/json.erb`
This template will output the collected data as a JSON document

#### Examples

##### 

```puppet
## Create a file from multiple string content blocks, with ordering

collections::file { '/etc/motd':
  collector => 'motd',
  template  => 'collections/concat.erb',
  data      => {
    order   => 1,
    content => file('my-module/motd-header'),
  },
  file      => {
    owner   => 'root',
    group   => 'root',
    mode    => '0444',
  },
}
collections::file::fragment { 'Add an unauthorised access warning':
  target    => 'motd',
  data      => {
    order   => 2,
    content => file('my-module/motd-unathorised-warning'),
  },
}

## Create a yaml file using multiple merged values

collections::file { '/etc/service/config.yaml':
  collector => 'service-config',
  template  => 'collections/yaml.erb',
  file      => {
    owner   => 'root',
    group   => 'service',
    mode    => '0640',
  },
}
collections::file::fragment { 'Set the user and group':
  target  => 'service-config',
  data    => {
    user  => 'serviceuser',
    group => 'serviceuser',
  },
}
```

#### Parameters

The following parameters are available in the `collections::file` defined type:

* [`collector`](#-collections--file--collector)
* [`template`](#-collections--file--template)
* [`data`](#-collections--file--data)
* [`file`](#-collections--file--file)
* [`merge_options`](#-collections--file--merge_options)
* [`reverse_merge_order`](#-collections--file--reverse_merge_order)

##### <a name="-collections--file--collector"></a>`collector`

Data type: `String[1]`

The name of this collection

##### <a name="-collections--file--template"></a>`template`

Data type: `String[3]`

The name of a template to use. This should be a String in the form of `modulename/filename`.

##### <a name="-collections--file--data"></a>`data`

Data type: `Any`

Optional. Initial data for the collection.
If provided, a `collection::append` resource named `<collectionname>::auto-initial-data` will
be created.

Default value: `undef`

##### <a name="-collections--file--file"></a>`file`

Data type: `Hash[String, Any]`

Parameters to pass to the `file` resource which will be created by this collection.
Most values will be passed through, but `source` or `content` will be removed
(As allowing them would be ambiguous) and `ensure` will only be allowed if set to
`absent`, `file` or `present`.

Default value: `{}`

##### <a name="-collections--file--merge_options"></a>`merge_options`

Data type: `Hash[String,Variant[Boolean,String]]`

Default: { keep_array_duplicates => true } (for compatability with datacat)
Options to pass to the Ruby `deep_merge` gem. See the [options reference](https://github.com/danielsdeleo/deep_merge?tab=readme-ov-file#options) for details.

Default value: `{}`

##### <a name="-collections--file--reverse_merge_order"></a>`reverse_merge_order`

Data type: `Boolean`

Default: false
Give merge priority to items later in the set

Default value: `false`

### <a name="collections--register_action"></a>`collections::register_action`

In the collections module, an `action` is a defined_type that will be called
for each item added to the collection, passed in as the `item` parameter

#### Examples

##### 

```puppet
collections::create { 'admin_users': }

collections::register_action { 'Admin users get database access':
  target     => admin_users,
  resource   => 'database::add_admin_access',
  parameters => {
    require => Service['database']
  }
}

collections::append { 'Alice is an admin user':
  target => admin_users,
  data   => {
    user => 'Alice',
    uid  => 1001,
    home => '/home/alice'
  }
}

collections::append { 'Bob is an admin user':
  target => admin_users,
  data   => {
    user => 'Bob',
    uid  => 1002,
    home => '/home/bob'
  }
}

# Will result in:
database::add_admin_access { 'admin_users::1',
  target => admin_users,
  item   => {
    user => 'Alice',
    uid  => 1001,
    home => '/home/alice'
  },
  require => Service['database']
}
database::add_admin_access { 'admin_users::2',
  target => admin_users,
  item   => {
    user => 'Bob',
    uid  => 1002,
    home => '/home/bob'
  },
  require => Service['database']
}
```

#### Parameters

The following parameters are available in the `collections::register_action` defined type:

* [`target`](#-collections--register_action--target)
* [`resource`](#-collections--register_action--resource)
* [`parameters`](#-collections--register_action--parameters)

##### <a name="-collections--register_action--target"></a>`target`

Data type: `String[1]`

The name of the collection to configure

##### <a name="-collections--register_action--resource"></a>`resource`

Data type: `String[1]`

The name of a defined_type that will be created once per item

##### <a name="-collections--register_action--parameters"></a>`parameters`

Data type: `Hash[String,Any]`

Optional parameters that will be passed to the action resource when created

Default value: `{}`

### <a name="collections--register_executor"></a>`collections::register_executor`

In the collections module, an `executor` is a defined_type that will be called
once, with all items in the collection passed to it as the `items` parameter

#### Examples

##### 

```puppet
collections::create { 'admin_users': }

collections::register_executor { 'Admin users get database access':
  target     => admin_users,
  resource   => 'database::add_admin_access',
  parameters => {
    require => Service['database']
  }
}

collections::append { 'Alice is an admin user':
  target => admin_users,
  data   => {
    user => 'Alice',
    uid  => 1001,
    home => '/home/alice'
  }
}

collections::append { 'Bob is an admin user':
  target => admin_users,
  data   => {
    user => 'Bob',
    uid  => 1002,
    home => '/home/bob'
  }
}

# Will result in:
database::add_admin_access { 'admin_users::executor',
  target => admin_users,
  items  => [
    {
      user => 'Alice',
      uid  => 1001,
      home => '/home/alice',
    },
    {
      user => 'Bob',
      uid  => 1002,
      home => '/home/bob',
    },
  },
  require => Service['database']
}
```

#### Parameters

The following parameters are available in the `collections::register_executor` defined type:

* [`target`](#-collections--register_executor--target)
* [`resource`](#-collections--register_executor--resource)
* [`parameters`](#-collections--register_executor--parameters)

##### <a name="-collections--register_executor--target"></a>`target`

Data type: `String[1]`

The name of the collection to configure

##### <a name="-collections--register_executor--resource"></a>`resource`

Data type: `String[1]`

The name of a defined_type

##### <a name="-collections--register_executor--parameters"></a>`parameters`

Data type: `Hash[String,Any]`

Optional parameters that will be passed to the action resource when created

Default value: `{}`

### <a name="collections--tap"></a>`collections::tap`

This is a debugging tool and example action. It creates a
notify resource for the item passed in to it.

#### Examples

##### 

```puppet
collections::create { 'foo': }

collections::append { 'foo A':
  target => 'foo',
  data   => 'A'
}

collections::append { 'foo B':
  target => 'foo',
  data   => 'B'
}

collections::register_action { 'Debug: Print the items'

# Will result in:
collections::tap { 'foo:1':
  target => 'foo',
  item   => 'A'
}
collections::tap { 'foo:2':
  target => 'foo',
  item   => 'B'
}
```

#### Parameters

The following parameters are available in the `collections::tap` defined type:

* [`target`](#-collections--tap--target)
* [`item`](#-collections--tap--item)

##### <a name="-collections--tap--target"></a>`target`

Data type: `String[1]`

Passed in by `collections::commit` when creating this resource. It indicates the
name of the collection that it was spawned from, to allow any

##### <a name="-collections--tap--item"></a>`item`

Data type: `Any`

Passed in by `collections::commit` when creating this resource. It contains an
one item from the collection.

## Functions

### <a name="collections--deep_merge"></a>`collections::deep_merge`

Type: Ruby 4.x API

Returns a copy of dest with source merged into it

#### `collections::deep_merge(Any $dest, Any $source, Optional[Hash[String, Variant[Boolean,String]]] $options)`

Returns a copy of dest with source merged into it

Returns: `Any`

##### `dest`

Data type: `Any`

The destination object, which will be overridden by the contents of source

##### `source`

Data type: `Any`

The source object, which will override the contents of dest

##### `options`

Data type: `Optional[Hash[String, Variant[Boolean,String]]]`

Options for the deep_merge method.

